from django.db import models
from simple_history.models import HistoricalRecords

class Storage(models.Model):
    array = models.CharField(max_length=512)
    capacity = models.DecimalField(max_digits=10, decimal_places=2)
    history = HistoricalRecords()
    class Meta:
        verbose_name_plural = 'Storage Arrays'
    def __unicode__(self):
        return self.array

class ClusterStorage(models.Model):
    name = models.CharField(max_length=256)
    storage = models.ForeignKey(Storage)
    capacity = models.IntegerField()
    history = HistoricalRecords()
    class Meta:
        verbose_name_plural = 'Cluster Assigned Storage'
    def __unicode__(self):
        return self.name

class Ssd(models.Model):
    name = models.CharField(max_length=256)
    type = models.CharField(max_length=256, verbose_name='Type')
    vendor = models.CharField(max_length=256)
    capacity = models.IntegerField(verbose_name='Capacity in GB')
    wwn = models.CharField(max_length=256, blank=True, verbose_name='WWN')
    history = HistoricalRecords()
    class Meta:
        verbose_name_plural = 'SSD'
    def __unicode__(self):
        return self.name

class Server(models.Model):
    name = models.CharField(max_length=256)
    fqdn = models.CharField(max_length=512)
    cores = models.IntegerField()
    memory = models.IntegerField()
    processor = models.CharField(max_length=512)
    archetecture = models.CharField(max_length=512)
    kernel = models.CharField(max_length=512)
    os = models.CharField(max_length=512)
    ssd = models.ManyToManyField(Ssd, blank=True)
    history = HistoricalRecords()
    class Meta:
        ordering = ('name',)
    def __unicode__(self):
        return self.name

class Cluster(models.Model):
    name = models.CharField(max_length=256)
    owner = models.CharField(max_length=256, blank=True)
    description = models.CharField(max_length=512)
    nodes = models.ManyToManyField(Server, blank=True)
    storage = models.ManyToManyField(ClusterStorage, blank=True)
    history = HistoricalRecords()
    def __unicode__(self):
        return self.name

# Create your models here.
